package com.lj360.memberdata;

import org.codehaus.jackson.map.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.boot.web.servlet.ServletComponentScan;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.client.loadbalancer.LoadBalanced;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.BeanIds;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.NoOpPasswordEncoder;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CorsFilter;

@SpringBootApplication
@ServletComponentScan
@EnableDiscoveryClient
@EnableJpaRepositories(basePackages = "com.lj360.memberdata")
public class AuthServerApplication extends WebSecurityConfigurerAdapter {

    public static void main(String[] args) {
        SpringApplication.run(AuthServerApplication.class, args);
    }

    @LoadBalanced
    @Bean
    RestTemplate restTemplate() {
        return new RestTemplate();
    }

    @Bean
    public ObjectMapper getObjectMapper(){
        ObjectMapper objectMapper = new ObjectMapper();
        return objectMapper;
    }
    @Autowired
    private UserDetailsService userDetailsService;

    @Autowired
    protected AuthenticationSuccessHandler ssbAuthenticationSuccessHandler;

    @Autowired
    protected AuthenticationFailureHandler ssbAuthenticationFailureHandler;

//    @Autowired
//    private UserNameAuthenticationFilter filter;

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http.csrf().disable();
        http
                .requestMatchers().antMatchers("/oauth/**","/login/**","/logout/**")
                .and()
                .authorizeRequests()
                .antMatchers("/oauth/**").authenticated()
                .and()
                .formLogin()
                .loginPage("http://localhost:8000/login")           // 设置登录页面 app接口用不到
                .loginProcessingUrl("/login")  // 自定义的登录接口
				.successHandler(ssbAuthenticationSuccessHandler)//成功处理器
				.failureHandler(ssbAuthenticationFailureHandler)//失败处理器
                .permitAll()//新增login form支持用户登录及授权
                .and()
                .headers().frameOptions().disable()
                .and()
                .cors()// 跨域需要添加此配置项
//                .and()
//                .addFilterBefore(myAuthenticationFilter(), UsernamePasswordAuthenticationFilter.class)

                ;
    }

//    @Bean
//    public FilterRegistrationBean someFilterRegistration() throws Exception{
//        FilterRegistrationBean registration = new FilterRegistrationBean();
//        registration.setFilter(myAuthenticationFilter());
//        registration.addUrlPatterns("/*");
//        registration.setName("commonDataFilter");
//        registration.setOrder(1);
//        return registration;
//    }
//
//    @Bean
//    UserNameAuthenticationFilter myAuthenticationFilter() throws Exception {
//        UserNameAuthenticationFilter filter = new UserNameAuthenticationFilter();
//        filter.setAuthenticationManager(authenticationManagerBean());
//        filter.setAuthenticationSuccessHandler(ssbAuthenticationSuccessHandler);
//        filter.setAuthenticationFailureHandler(ssbAuthenticationFailureHandler);
//        return filter;
//    }

    @Bean(name = BeanIds.AUTHENTICATION_MANAGER)
    @Override
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }

    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
//        auth
//        .inMemoryAuthentication()
//        .withUser("guest").password("guest")
//                .authorities("WRIGTH_READ")
//        .and()
//        .withUser("admin").password("admin")
//        .authorities("WRIGTH_READ", "WRIGTH_WRITE");

        //自定义验证用户
        auth.eraseCredentials(false)
                .userDetailsService(userDetailsService);
    }

    @Bean
    public static NoOpPasswordEncoder passwordEncoder() {
      return (NoOpPasswordEncoder) NoOpPasswordEncoder.getInstance();
    }
//
//    @Bean
//    public static PasswordEncoder passwordEncoder() {
//        return new BCryptPasswordEncoder();
//    }

    @Bean
    public CorsFilter corsFilter() {
        final UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        final CorsConfiguration config = new CorsConfiguration();
        config.setAllowCredentials(true);
        config.addAllowedOrigin("*");
        config.addAllowedHeader("*");
        config.addAllowedMethod("OPTIONS");
        config.addAllowedMethod("HEAD");
        config.addAllowedMethod("GET");
        config.addAllowedMethod("PUT");
        config.addAllowedMethod("POST");
        config.addAllowedMethod("PATCH");
        source.registerCorsConfiguration("/**", config);
        config.addAllowedMethod("DELETE");
        return new CorsFilter(source);
    }
}
